local KeyMapper = import('/lua/keymap/keymapper.lua')
local Prefs = import('/lua/user/prefs.lua')

function Init()
	KeyMapper.SetUserKeyAction('toggle_repeat_build', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").ToggleRepeatBuild()', category = 'user', order = 1})
	KeyMapper.SetUserKeyAction('show_enemy_life', {action = 'UI_ForceLifbarsOnEnemy', category = 'user', order = 2})
	KeyMapper.SetUserKeyAction('show_network_stats', {action =  'ren_ShowNetworkStats', category = 'user', order = 3})
	--add key to create an upgrade marker
	KeyMapper.SetUserKeyAction('scu_upgrade_marker', {action =  'UI_Lua import("/mods/gui/modules/scumanager.lua").CreateMarker()', category = 'user', order = 4})
	KeyMapper.SetUserKeyAction('toggle_shield', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleScript("Shield")', category = 'user', order = 6})
	KeyMapper.SetUserKeyAction('toggle_weapon', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleScript("Weapon")', category = 'user', order = 7})
	KeyMapper.SetUserKeyAction('toggle_jamming', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleScript("Jamming")', category = 'user', order = 8})
	KeyMapper.SetUserKeyAction('toggle_intel', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleScript("Intel")', category = 'user', order = 9})
	KeyMapper.SetUserKeyAction('toggle_production', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleScript("Production")', category = 'user', order = 10})
	KeyMapper.SetUserKeyAction('toggle_stealth', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleScript("Stealth")', category = 'user', order = 11})
	KeyMapper.SetUserKeyAction('toggle_generic', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleScript("Generic")', category = 'user', order = 12})
	KeyMapper.SetUserKeyAction('toggle_special', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleScript("Special")', category = 'user', order = 13})
	KeyMapper.SetUserKeyAction('toggle_cloak', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleScript("Cloak")', category = 'user', order = 14})
	KeyMapper.SetUserKeyAction('toggle_all', {action = 'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleAllScript()', category = 'user', order = 15})
	KeyMapper.SetUserKeyAction('teleport', {action =  'StartCommandMode order RULEUCC_Teleport', category = 'user', order = 5})
	KeyMapper.SetUserKeyAction('military_overlay', {action =  'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleOverlay("Military")', category = 'user', order = 16})
	KeyMapper.SetUserKeyAction('intel_overlay', {action =  'UI_Lua import("/mods/GUI/modules/keymapping.lua").toggleOverlay("Intel")', category = 'user', order = 17})
end

function ToggleRepeatBuild()
	local selection = GetSelectedUnits()
	if selection then
		local allFactories = true
		local currentInfiniteQueueCheckStatus = false
		for i,v in selection do
			if v:IsRepeatQueue() then
				currentInfiniteQueueCheckStatus = true
			end
			if not v:IsInCategory('FACTORY') then
				allFactories = false
			end
		end
		if allFactories then
			for i,v in selection do
				if currentInfiniteQueueCheckStatus then
					v:ProcessInfo('SetRepeatQueue', 'false')
				else
					v:ProcessInfo('SetRepeatQueue', 'true')
				end
			end
		end
	end
end

--function to toggle things like shields etc
-- Unit toggle rules copied from orders.lua, used for converting to the numbers needed for the togglescriptbit function
unitToggleRules = {
    Shield =  0,
    Weapon = 1,
    Jamming = 2,
    Intel = 3,
    Production = 4,
    Stealth = 5,
    Generic = 6,
    Special = 7,
    Cloak = 8,}

function toggleScript(name)
	local selection = GetSelectedUnits()
	local number = unitToggleRules[name]
	local currentBit = GetScriptBit(selection, number)
	ToggleScriptBit(selection, number, currentBit)
end

function toggleAllScript(name)
	local selection = GetSelectedUnits()
	for i = 0,8 do
		local currentBit = GetScriptBit(selection, i)
		ToggleScriptBit(selection, i, currentBit)
	end
end

local MilitaryFilters = {"Defense","AntiNavy","Miscellaneous","AntiAir","DirectFire","IndirectFire"}
local IntelFilters = {"CounterIntel", "Omni", "Radar", "Sonar"}

function toggleOverlay(type)
	local currentFilters = Prefs.GetFromCurrentProfile('activeFilters') or {}
	local tempFilters = {}
	local MilitaryActive = false
	local IntelActive = false
	for i, filter in MilitaryFilters do
		if currentFilters[string.lower(filter)] then
			MilitaryActive = true
		end
	end
	for i, filter in IntelFilters do
		if currentFilters[string.lower(filter)] then
			IntelActive = true
		end
	end
	
	local function toggleFilters(filterTable, active)
		for i, filter in filterTable do
			if active then
				currentFilters[string.lower(filter)] = nil
			else
				currentFilters[string.lower(filter)] = true
				table.insert(tempFilters, filter)
			end
		end
	end
	
	if type == 'Military' then
		toggleFilters(MilitaryFilters, MilitaryActive)
		if IntelActive then
			for i, filter in IntelFilters do
				table.insert(tempFilters, filter)
			end
		end
	end
	
	if type == 'Intel' then
		toggleFilters(IntelFilters, IntelActive)
		if MilitaryActive then
			for i, filter in MilitaryFilters do
				table.insert(tempFilters, filter)
			end
		end
	end
	
	Prefs.SetToCurrentProfile('activeFilters', currentFilters)
	import('/lua/ui/game/multifunction.lua').UpdateActiveFilters()
end
